@@ -5,7 +5,11 @@ import sys
 import pathlib
 
 import ROOT
+ROOT.gROOT.SetBatch(True) 
 ROOT.PyConfig.IgnoreCommandLineOptions = True
 macro_path = pathlib.Path(__file__).parent.absolute()
-ROOT.gSystem.Load(os.path.join(macro_path, 'macros', 'RooTwoSidedCBShape_cxx'))
-from ROOT import RooTwoSidedCBShape
\ No newline at end of file
+try:
+    ROOT.gSystem.Load(os.path.join(macro_path, 'macros', 'RooTwoSidedCBShape_cxx'))
+    from ROOT import RooTwoSidedCBShape
+except Exception:
+    pass
\ No newline at end of file
