@@ -11,7 +11,7 @@ import numpy as np
 
 from quickstats.components import CLIParser, NuisanceParameterPull
 from quickstats.components.root_plots import StandardGraphMaker, set_attrib
-from quickstats.components.numerics import array_swap, reorder_arrays
+from quickstats.components.numerics import array_swap, reorder_arrays, reverse_arrays
 from quickstats.utils import root_utils
 
 import ROOT
@@ -32,8 +32,8 @@ class NPRankingPlotter(object):
     _DEFAULT_PLOT_CONFIG_ = {
         'num_param'    : -1,
         'threshold'    : 0.0,
-        'scale_poi'    : 10.0,
-        'scale_theta'  : 10.0,
+        'scale_poi'    : 1.0,
+        'scale_theta'  : 1.0,
         'show_names'   : True,
         'show_prefit'  : True,
         'show_postfit' : True,
@@ -102,14 +102,14 @@ class NPRankingPlotter(object):
                 'description' : 'Scaling of impact axis',
                 'required'    : False,
                 'type'        : float,
-                'default'     : 10.0
+                'default'     : 1.0
             },          
             'scale_theta': {
                 'abbr'        : None,
                 'description' : 'Scaling of pull axis',
                 'required'    : False,
                 'type'        : float,
-                'default'     : 10.0
+                'default'     : 1.0
             },   
             'show_names': {
                 'abbr'        : None,
@@ -133,14 +133,14 @@ class NPRankingPlotter(object):
                 'default'     : 1
             },   
             'relative': {
-                'abbr'        : 'r',
+                'abbr'        : None,
                 'description' : 'Show relative variation',
                 'required'    : False,
                 'type'        : int,
                 'default'     : 0
             },    
             'correlation': {
-                'abbr'        : 'c',
+                'abbr'        : None,
                 'description' : 'Visualize sign of correlation',
                 'required'    : False,
                 'type'        : int,
@@ -154,7 +154,7 @@ class NPRankingPlotter(object):
                 'default'     : 1
             },        
             'rank': {
-                'abbr'        : 'r',
+                'abbr'        : None,
                 'description' : 'Order nuisances by impact',
                 'required'    : False,
                 'type'        : int,
@@ -180,7 +180,14 @@ class NPRankingPlotter(object):
                 'required'    : False,
                 'type'        : str,
                 'default'     : ""
-            }          
+            },
+            'out_dir': {
+                'abbr'        : 'o',
+                'description' : 'Directory where the output files are created',
+                'required'    : False,
+                'type'        : str,
+                'default'     : "./"
+            },        
         }
     
     @property
@@ -190,29 +197,9 @@ class NPRankingPlotter(object):
     @property
     def color_scheme(self):
         return self._color_scheme
-    
-    @property
-    def canvas(self):
-        return self._canvas
-    
-    @property
-    def pad(self):
-        return self._pad
 
-    def __init__(self):
-        
+    def __init__(self):   
         self.configure()
-        
-        if not ROOT.gROOT.FindObject("c1"):
-            self._canvas = ROOT.TCanvas("c1", "c1", 1500, 2000)
-        else:
-            self._canvas = ROOT.gROOT.FindObject("c1")
-        if not ROOT.gROOT.FindObject("pad1"): 
-            self._pad    = ROOT.TPad("pad1", "pad1", 0.0, 0.0, 1.0, 1.0, 0)
-        else:
-            self._pad    = ROOT.gROOT.FindObject("pad1")
-            
-        self.setup_canvas()
 
     def get_parser(self, **kwargs):
         parser = CLIParser(description=self._CLI_DESCRIPTION_,
@@ -223,7 +210,7 @@ class NPRankingPlotter(object):
     def run_parser(self, args=None):
         parser = self.get_parser()
         kwargs = vars(parser.parse_args(args))
-        self.plot_pulls(**kwargs)   
+        self.run_plot(**kwargs)   
         
     def configure(self, **kwargs):
         self._config = self._DEFAULT_PLOT_CONFIG_
@@ -301,19 +288,20 @@ class NPRankingPlotter(object):
             'poi_postfit_anticorr':{
                 'LineColor'        : ROOT.TColor.GetColor(kColorAntiCorr),
                 'FillColor'        : ROOT.TColor.GetColor(kColorAntiCorr),
-                'LineWidth'        : 1,
+                'LineWidth'        : 0,
                 'MarkerSize'       : 0,
             },
             'poi_postfit_anticorr_ol':{
                 'LineColor'        : ROOT.TColor.GetColor(kColorAntiCorrOL),
                 'FillColor'        : ROOT.TColor.GetColor(kColorAntiCorrOL),
-                'LineWidth'        : 1,
+                'LineWidth'        : 0,
                 'MarkerSize'       : 0,
             },            
             'poi_prefit_corr':{
                 'LineColor'        : ROOT.TColor.GetColor(kColorPrefit),
                 'FillColor'        : ROOT.TColor.GetColor(kColorPrefit),
-                'LineWidth'        : 0,
+                'FillStyle'        : 0,
+                'LineWidth'        : 1,
                 'MarkerSize'       : 0,
             },
             'poi_prefit_ol_corr':{
@@ -345,8 +333,6 @@ class NPRankingPlotter(object):
                 'MarkerSize'       : 0
             },
             'label':{
-                'LabelsOption'     : 'h',
-                'LabelSize'        : 0.02,
                 'Xaxis.LabelColor' : ROOT.kWhite,
                 'Xaxis.AxisColor'  : ROOT.kWhite,
                 'Yaxis.LabelColor' : ROOT.kBlack,
@@ -391,29 +377,39 @@ class NPRankingPlotter(object):
                 'NDC'              : None,
                 'TextFont'         : 72,
                 'TextColor'        : ROOT.kBlack,
-                'TextSize'         : 0.70
+                'TextSize'         : 0.035
             },
             'internal_label':{
                 'NDC'              : None,
                 'TextFont'         : 42,
                 'TextColor'        : 1,
-                'TextSize'         : 0.70
+                'TextSize'         : 0.035
             }            
         }
-        
         return plot_options                
         
     def setup_canvas(self):
-        if self.config['show_names']:
-            self.pad.SetLeftMargin(0.35)
-        else:
-            self.pad.SetLeftMargin(0.05)
-        self.pad.SetRightMargin(0.05)
-        self.pad.SetBottomMargin(0.09)
-        self.pad.SetTopMargin(0.09)
         
-        self.pad.Draw()
-        self.pad.cd()
+        if not ROOT.gROOT.FindObject("c1"):
+            canvas = ROOT.TCanvas("c1", "c1", 1500, 2000)
+        else:
+            canvas = ROOT.gROOT.FindObject("c1")
+        if not ROOT.gROOT.FindObject("pad1"): 
+            pad    = ROOT.TPad("pad1", "pad1", 0.0, 0.0, 1.0, 1.0, 0)
+        else:
+            pad    = ROOT.gROOT.FindObject("pad1")
+            
+        if self.config['show_names']:
+            pad.SetLeftMargin(0.35)
+        else:
+            pad.SetLeftMargin(0.05)
+        pad.SetRightMargin(0.05)
+        pad.SetBottomMargin(0.09)
+        pad.SetTopMargin(0.09)
+        
+        pad.Draw()
+        pad.cd()
+        return canvas, pad
         
     @staticmethod
     def extract_pull_results(input_dir:str, poi:str):
@@ -455,9 +451,9 @@ class NPRankingPlotter(object):
         return pull_results
             
     def run_plot(self, input_dir:str, poi:str, overlay_dir:str='', num_param:int=-1, threshold:float=0.0,
-                 scale_poi:float=10.0, scale_theta:float=10.0, show_names:bool=True, show_prefit:bool=True,
+                 scale_poi:float=1.0, scale_theta:float=1.0, show_names:bool=True, show_prefit:bool=True,
                  show_postfit:bool=True, relative:bool=False, correlation:bool=True, onesided:bool=True,
-                 rank:bool=True, log_level:str="INFO", map_param:str="", label:str=""):
+                 rank:bool=True, log_level:str="INFO", map_param:str="", label:str="", out_dir:str="./"):
         
         data = self.extract_pull_results(input_dir, poi)
         data_overlay = None
@@ -496,7 +492,8 @@ class NPRankingPlotter(object):
             keep_indices = np.in1d(data_overlay['nuis_label'], labels_to_keep)
             for key in data_overlay:
                 data_overlay[key] = data_overlay[key][keep_indices]
-        self.create_ranking_plot(data, data_overlay, param_dict)
+        #return data, data_overlay
+        self.create_ranking_plot(data, data_overlay, param_dict, out_dir=out_dir)
     
     @staticmethod
     def pad_default_impact(source, pad_labels:Optional[List]=None, 
@@ -540,15 +537,21 @@ class NPRankingPlotter(object):
 
         if (not correlation) or (not onesided):
             array_swap(impact_up, impact_down, impact_up < 0)
-
+        # secondary rows for one-sided impacts
+        impact_up_sec = deepcopy(impact_up)
+        impact_down_sec = deepcopy(impact_down)
         if onesided:
             indices = (impact_up > 0) & (impact_down > 0)
-            impact_up[indices] = impact_down[indices]
+            impact_up[indices] = impact_down_sec[indices]
             impact_down[indices] = 0
+            impact_down_sec[indices] = 0
             indices = (impact_up < 0) & (impact_down < 0)
-            impact_down[indices] = impact_up[indices]
+            impact_down_sec[indices] = impact_up[indices]
             impact_up[indices] = 0
+            impact_up_sec[indices] = 0
         impact_up[:], impact_down[:] = abs(impact_up), abs(impact_down)
+        impact_up_sec[:], impact_down_sec[:] = abs(impact_up_sec), abs(impact_down_sec)
+        return impact_up_sec, impact_down_sec
     
     @staticmethod
     def process_data(data, scale_theta:float=1.0, scale_poi:float=1.0, correlation:bool=True,
@@ -556,34 +559,52 @@ class NPRankingPlotter(object):
                      num_param:int=-1, pad_labels:Optional[List]=None, **kwargs):
         source = deepcopy(data)
         # subtract poi_hat offset from impacts
-        for key in ['poi_up', 'poi_down', 'poi_nom_up', 'poi_nom_up']:
+        for key in ['poi_up', 'poi_down', 'poi_nom_up', 'poi_nom_down']:
             source[key] -= source['poi_hat']   
         anticorrelated     = source['nuis_label'][(source['poi_up'] < 0) & (source['poi_down'] > 0)]
         anticorrelated_nom = source['nuis_label'][(source['poi_nom_up'] < 0) & (source['poi_nom_down'] > 0)]
-        NPRankingPlotter.process_impact(source['poi_up'], source['poi_down'], correlation, onesided)
-        NPRankingPlotter.process_impact(source['poi_nom_up'], source['poi_nom_down'], correlation, onesided)
-        max_poi = max(max(source['poi_up']), max(source['poi_down']))
+        poi_up_sec, poi_down_sec = NPRankingPlotter.process_impact(
+            source['poi_up'], source['poi_down'], correlation, onesided)
+        source['poi_up_sec'] = poi_up_sec
+        source['poi_down_sec'] = poi_down_sec
+        poi_nom_up_sec, poi_nom_down_sec = NPRankingPlotter.process_impact(
+            source['poi_nom_up'], source['poi_nom_down'], correlation, onesided)
+        source['poi_nom_up_sec'] = poi_nom_up_sec
+        source['poi_nom_down_sec'] = poi_nom_down_sec        
+        max_poi = max(max(source['poi_up']), max(source['poi_down']),
+                      max(source['poi_up_sec']), max(source['poi_down_sec']))
         NPRankingPlotter.pad_default_impact(source, pad_labels)
         if ranking:
+            impact = (source['poi_up'] + source['poi_down'] +
+                     source['poi_up_sec'] + source['poi_down_sec'])
+            impact /= (1*(source['poi_up'] != 0) + 1*(source['poi_down'] != 0) +
+                       1*(source['poi_up_sec'] != 0) + 1*(source['poi_down_sec'] != 0))
             # order by impact, from largest to smallest
-            reorder_arrays(source['poi_up'] + source['poi_down'], *source.values())
+            reorder_arrays(impact, *source.values())
         else:
             # order by NP names
             reorder_arrays(source['nuis_label'], *source.values(), descending=False)
+
         NPRankingPlotter.remove_impact_data(source, threshold, num_param)
-        for key in ['poi_up', 'poi_down', 'poi_nom_up', 'poi_nom_down']:
+        
+        # rescaling impacts and pulls
+        for key in ['poi_up', 'poi_down', 'poi_nom_up', 'poi_nom_down',
+                    'poi_up_sec', 'poi_down_sec', 'poi_nom_up_sec', 'poi_nom_down_sec']:
             source[key] *= (scale_poi / max_poi)
             if relative:
                 source[key] /= source['poi_hat']
-
         source['nuis_val'] *= scale_theta
         for key in ['nuis_up', 'nuis_down']:
             source[key] /= source['nuis_prefit']   
             source[key] = abs(source[key])*scale_theta
+    
         source['poi_corr']     = ~np.in1d(source['nuis_label'], anticorrelated)
         source['poi_nom_corr'] = ~np.in1d(source['nuis_label'], anticorrelated_nom)
         # dirty fix for carrying max_poi inside source
         source['max_poi'] = np.array([max_poi]*len(source['nuis_label']))
+        if ranking:
+            # order by impact, from smallest to largest
+            reverse_arrays(*source.values())
         return source
     
     @staticmethod
@@ -598,10 +619,17 @@ class NPRankingPlotter(object):
         impacts['up_anticorr'][correlated_indices]   = 0.0
         impacts['down_corr'][~correlated_indices]    = 0.0
         impacts['down_anticorr'][correlated_indices] = 0.0
-        for key in impacts:
-            impacts[key] = np.vstack((impacts[key], impacts[key])).ravel('F')
+        #for key in impacts:
+        #    impacts[key] = np.vstack((impacts[key], impacts[key])).ravel('F')
         return impacts
-
+    
+    @staticmethod
+    def _combine_impacts(primary, secondary):
+        impacts = {}
+        for key in ['up_corr', 'up_anticorr', 'down_corr', 'down_anticorr']:
+            impacts[key] = np.vstack((primary[key], secondary[key])).ravel('F')
+        return impacts
+    
     def get_plot_data(self, data, data_overlay=None):
         
         n_nuis = len(data['nuis_label'])
@@ -618,14 +646,25 @@ class NPRankingPlotter(object):
         cen_up_ol   = np.array([self._BOX_HEIGHT_OL_]*2*n_nuis)
         cen_down_ol = np.array([self._BOX_HEIGHT_OL_]*2*n_nuis)        
              
-        poi_impacts     = self.split_impact_by_correlation(data['poi_up'], 
-                                                           data['poi_down'],
-                                                           data['poi_corr'])
-
-        poi_nom_impacts = self.split_impact_by_correlation(data['poi_nom_up'],
-                                                           data['poi_nom_down'],
-                                                           data['poi_nom_corr'])
-
+        primary     = self.split_impact_by_correlation(data['poi_up'], 
+                                                       data['poi_down'],
+                                                       data['poi_corr'])
+        
+        secondary   = self.split_impact_by_correlation(data['poi_up_sec'], 
+                                                       data['poi_down_sec'],
+                                                       data['poi_corr'])        
+        poi_impacts = self._combine_impacts(primary, secondary)
+        
+        nom_primary = self.split_impact_by_correlation(data['poi_nom_up'],
+                                                       data['poi_nom_down'],
+                                                       data['poi_nom_corr'])
+        
+        nom_secondary = self.split_impact_by_correlation(data['poi_nom_up_sec'], 
+                                                         data['poi_nom_down_sec'],
+                                                         data['poi_nom_corr'])   
+        poi_nom_impacts = self._combine_impacts(nom_primary, nom_secondary)
+        #from pdb import set_trace
+        #set_trace()
         if data_overlay:
             nuis_y = np.array(range(n_nuis)) + 0.75
             poi_y  = np.array([0.625, 0.875] * n_nuis) + np.vstack((range(n_nuis), range(n_nuis))).ravel('F')
@@ -752,13 +791,13 @@ class NPRankingPlotter(object):
             plot_data['poi_prefit_ol_anticorr']  = None
             plot_data['poi_postfit_ol_corr']     = None
             plot_data['poi_postfit_ol_anticorr'] = None
-        n_shade = math.ceil((n_nuis + 1)/2)
+        n_shade = (n_nuis + 1)//2
         plot_data['shade'] = {
             'n'      : n_shade,
             'x'      : np.zeros(n_shade),
             'y'      : np.array(range(n_shade))*2 + 0.5,
-            'xerr_lo': np.array([100]*n_shade),
-            'xerr_hi': np.array([100]*n_shade),
+            'xerr_lo': np.array([100.]*n_shade),
+            'xerr_hi': np.array([100.]*n_shade),
             'yerr_lo': np.array([0.5]*n_shade),
             'yerr_hi': np.array([0.5]*n_shade)
         }
@@ -814,34 +853,35 @@ class NPRankingPlotter(object):
         }
         return plot_data
 
-    def create_ranking_plot(self, data, data_overlay=None, param_dict=None):
+    def create_ranking_plot(self, data, data_overlay=None, param_dict=None, out_dir="./"):
         
-        self.pad.cd()
+        canvas, pad = self.setup_canvas()
         n_nuis = len(data['nuis_label'])
         plot_data = self.get_plot_data(data, data_overlay)  
         plot_options = self.get_plot_options()
         
         # format histogram for labels
-        hist = StandardGraphMaker.TH2F("h_ranking", "h_ranking", **plot_data['label'])
+        hist = StandardGraphMaker.TH2F("h_ranking", "", **plot_data['label'])
         labels = data['nuis_label'] if self.config['show_names'] else np.array([""]*n_nuis)
         if param_dict is not None:
             labels = np.array([param_dict[label] for label in labels])
-        from pdb import set_trace
-        set_trace()
+        offset = math.ceil(2 * n_nuis / 10)
         for i, label in enumerate(labels):
-            hist.GetYaxis().SetBinLabel(i + 1, label)
+            hist.GetYaxis().SetBinLabel(i + offset + 1, label)
         label_size = (1.0 / n_nuis) if (1.0 / n_nuis) < 0.02 else 0.02
         hist.SetLabelSize(label_size, "Y")
         set_attrib(hist, **plot_options['label'])
         hist.Draw("h")
         
         # format axis for poi
-        axis_poi = StandardGraphMaker.TGaxis(**plot_data['axis_poi'], options=plot_options['axis_poi'])
+        axis_poi = StandardGraphMaker.TGaxis(**plot_data['axis_poi'])
         axis_poi.ImportAxisAttributes(hist.GetXaxis())
+        set_attrib(axis_poi, **plot_options['axis_poi'])
         
         # format axis for theta
-        axis_theta = StandardGraphMaker.TGaxis(**plot_data['axis_theta'], options=plot_options['axis_theta'])
+        axis_theta = StandardGraphMaker.TGaxis(**plot_data['axis_theta'])
         axis_theta.ImportAxisAttributes(hist.GetXaxis())
+        set_attrib(axis_theta, **plot_options['axis_theta'])
         
         # format axis for labels
         axis_label = StandardGraphMaker.TGaxis(**plot_data['axis_label'], options=plot_options['axis_label'])
@@ -862,7 +902,6 @@ class NPRankingPlotter(object):
             if data_overlay:
                 graphs_to_draw.append('poi_prefit_ol_corr')
                 graphs_to_draw.append('poi_prefit_ol_anticorr')
-        
         # draw graphs
         graphs = {}
         for name in graphs_to_draw:
@@ -890,7 +929,7 @@ class NPRankingPlotter(object):
             graphs[name] = StandardGraphMaker.GraphAsymErrors(**plot_data[name], options=plot_options[name])
             graphs[name].Draw("p")
         
-        self.canvas.SetLogy(0)
+        canvas.SetLogy(0)
         
         # draw legends
         legend = StandardGraphMaker.TLegend(**plot_data['legend'], options=plot_options['legend'])
@@ -902,31 +941,34 @@ class NPRankingPlotter(object):
             legend.AddEntry(graphs['one_sigma_box_ol'], "Alt 1 standard deviation", "l")    
         if self.config['show_prefit']:
             legend.AddEntry(graphs['poi_prefit_corr'], 
-                            "Prefit Impact on #hat{" + self.config['poi'] + "}", "lp")
+                            "Prefit Impact on #hat{" + self.config['poi'] + "}", "f")
             if data_overlay:
                 legend.AddEntry(graphs['poi_prefit_corr_ol'], 
-                                "Alt Prefit Impact on #hat{" + self.config['poi'] + "}", "lp")
+                                "Alt Prefit Impact on #hat{" + self.config['poi'] + "}", "f")
         if self.config['show_postfit']:
             legend.AddEntry(graphs['poi_postfit_corr'], 
-                            "Postfit Impact on #hat{" + self.config['poi'] + "}", "lp")
+                            "Postfit Impact on #hat{" + self.config['poi'] + "}", "f")
             if data_overlay:
                 legend.AddEntry(graphs['poi_postfit_corr_ol'], 
-                                "Alt Postfit Impact on #hat{" + self.config['poi'] + "}", "lp")
+                                "Alt Postfit Impact on #hat{" + self.config['poi'] + "}", "f")
         legend.Draw()
         
-        atlas_label = StandardGraphMaker.TLatex(options=plot_options['ATLAS_label'])
-        internal_label = StandardGraphMaker.TLatex(options=plot_options['internal_label'])
+        label_1 = StandardGraphMaker.TLatex(options=plot_options['ATLAS_label'])
+        label_2 = StandardGraphMaker.TLatex(options=plot_options['internal_label'])
         
-        atlas_label.DrawLatex(self._POSITION_LABEL_X_, self._POSITION_LABEL_Y_, "ATLAS")
-        atlas_label.DrawLatex(self._POSITION_LABEL_X_ + self._POSITION_LABEL_DX_,
+        label_1.DrawLatex(self._POSITION_LABEL_X_, self._POSITION_LABEL_Y_, "ATLAS")
+        label_2.DrawLatex(self._POSITION_LABEL_X_ + self._POSITION_LABEL_DX_,
                               self._POSITION_LABEL_Y_, "Internal")
-        atlas_label.DrawLatex(self._POSITION_LABEL_X_,
+        label_2.DrawLatex(self._POSITION_LABEL_X_,
                               self._POSITION_LABEL_Y_ - 1 * self._POSITION_LABEL_DY_,
                               self.config['label'])  
-        atlas_label.DrawLatex(self._POSITION_LABEL_X_,
+        label_2.DrawLatex(self._POSITION_LABEL_X_,
                               self._POSITION_LABEL_Y_ - 2 * self._POSITION_LABEL_DY_,
                               "Rank 1 to {}".format(n_nuis))         
         
         save_name = "ranking_{}_rank_{:04d}_to_{:04d}".format(self.config['poi'], 1, n_nuis)
-        for extension in ['eps', 'pdf', 'png']:
-            self.canvas.SaveAs("{}.{}".format(save_name, extension))
\ No newline at end of file
+        save_path = os.path.join(out_dir, save_name)
+        if not os.path.exists(out_dir):
+            os.makedirs(out_dir)
+        for extension in ['eps', 'pdf', 'png', 'C']:
+            canvas.SaveAs("{}.{}".format(save_path, extension))
\ No newline at end of file
