@@ -199,7 +199,7 @@ class NuisanceParameterPull(object):
                 'required'    : False,
                 'type'        : int,
                 'default'     : 0
-            },                 
+            }                          
         } 
     
     @property
@@ -276,7 +276,13 @@ class NuisanceParameterPull(object):
                    precision:float=0.001, eps:float=1.0, strategy:int=0, 
                    num_cpu:int=1, offset:int=1, optimize:int=2, eigen:bool=False,
                    fix_cache:bool=True, fix_multi:bool=True, max_calls:int=-1, 
-                   max_iters:int=-1, outdir:str='output', **kwargs):
+                   max_iters:int=-1, outdir:str='output', checkpoint=True, **kwargs):
+        outname_root = os.path.join(outdir, nuis_name + '.root')
+        outname_json = os.path.join(outdir, nuis_name + '.json')
+        #checkpointing:
+        if (os.path.exists(outname_root) and os.path.exists(outname_json)) and checkpoint:
+            print('INFO: Jobs already finished for the NP: {}. Skipping.'.format(nuis_name))
+            return None
         start_time = time.time()
         snapshot_names = snapshot.split(',')
         # load model
@@ -453,7 +459,7 @@ class NuisanceParameterPull(object):
                  data='combData', poi='', snapshot='nominalNuis', outdir='output', profile="",
                  fix='', minimizer_type='Minuit2', minimizer_algo='Migrad', num_cpu=1, binned=1,
                  precision=0.001, eps=1.0, log_level='INFO', eigen=0, strategy=0, fix_cache=1, fix_multi=1,
-                 offset=1, optimize=2, parameter="", max_calls=-1, max_iters=-1, parallel=0, **kwargs):
+                 offset=1, optimize=2, parameter="", max_calls=-1, max_iters=-1, parallel=0, checkpoint=True, **kwargs):
         
         start_time = time.time()
         
